# Generated by Django 4.0.7 on 2022-08-24 19:46

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="TaskLog",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("app_name", models.CharField(db_index=True, max_length=255)),
                ("exception", models.TextField()),
                ("parent_id", models.UUIDField(default=None, null=True)),
                ("priority", models.IntegerField(default=None, null=True)),
                ("retries", models.IntegerField()),
                ("received", models.DateTimeField(default=None, null=True)),
                ("runtime", models.FloatField(db_index=True, default=None, null=True)),
                ("started", models.DateTimeField(default=None, null=True)),
                (
                    "state",
                    models.IntegerField(
                        choices=[(1, "success"), (2, "retry"), (3, "failure")],
                        db_index=True,
                    ),
                ),
                ("task_id", models.UUIDField(db_index=True, default=uuid.uuid4)),
                ("task_name", models.CharField(db_index=True, max_length=255)),
                ("timestamp", models.DateTimeField(db_index=True)),
                ("traceback", models.TextField()),
            ],
        ),
    ]
