{% extends "admin/change_list.html" %}
{% load static %}
{% load i18n %}

{% block object-tools-items %}
{{ block.super }}
<li>
    <a href="{% url 'taskmonitor:admin_taskmonitor_reports' %}" class="btn btn-high">
        {% translate "Reports" %}
    </a>
</li>
<li>
    <a href="{% url 'taskmonitor:admin_taskmonitor_download_csv' %}" class="btn btn-high">
        {% translate "Download as CSV" %}
    </a>
</li>
{% endblock %}
